package com.ringlord.odfind;

import java.awt.Component;
import java.awt.Color;

import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.FocusListener;
import java.awt.event.FocusEvent;

import java.util.List;

import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.MutableComboBoxModel;
import javax.swing.BorderFactory;
import javax.swing.ListCellRenderer;

import javax.swing.border.Border;

import javax.swing.event.PopupMenuListener;
import javax.swing.event.PopupMenuEvent;

/**
 * <p>An editable JComboBox for String items. Typing items that are
 * not yet in the list (and pressing ENTER) will add them. Removing
 * all characters from a selected item (and pressing ENTER) removes
 * the item again.</p>
 *
 * @author K Udo Schuermann
 **/
public class ComboText
  extends JComboBox<String>
{
  public ComboText( final MutableComboBoxModel<String> model )
  {
    super( model );
    this.model = model;
    this.editor = getEditor();
    //    setRenderer( new Renderer() ); // doesn't work too well right now

    model.insertElementAt( "", curIndex=0 );

    editor.addActionListener( new ActionListener()
      {
        public void actionPerformed( final ActionEvent e )
        {
          editorAction();
        }
      } );
    editor.getEditorComponent().addFocusListener( new FocusListener()
      {
        public void focusGained( final FocusEvent e )
        {
        }
        public void focusLost( final FocusEvent e )
        {
          editorAction();
        }
      } );
    addActionListener( new ActionListener()
      {
        public void actionPerformed( final ActionEvent e )
        {
          if( pickingFromList )
            {
              String item = (String)getSelectedItem();

              final int size = model.getSize();
              for( int i=0; i<size; i++ )
                {
                  if( item.equals(model.getElementAt(i)) )
                    {
                      if( PROMOTE_SELECTION )
                        {
                          if( i > 1 )
                            {
                              // move the element to the top-spot
                              model.removeElementAt( i );
                              model.insertElementAt( item, curIndex=1 );
                              setSelectedItem( item );
                            }
                        }
                      else
                        {
                          curIndex = i;
                        }
                      break;
                    }
                }
            }
          pickingFromList = false;
        }
      } );
    addPopupMenuListener( new PopupMenuListener()
      {
        public void popupMenuCanceled( final PopupMenuEvent e )
        {
        }
        public void popupMenuWillBecomeVisible( final PopupMenuEvent e )
        {
          pickingFromList = true;
        }
        public void popupMenuWillBecomeInvisible( final PopupMenuEvent e )
        {
        }
      } );
  }

  public void finishEditing()
  {
    editorAction();
  }

  private void editorAction()
  {
    add( (String)editor.getItem() );
  }

  public void add( final String item )
  {
    if( "".equals(item) )
      {
        // user blanked out a previously existing item, so we'll
        // delete it from the list
        if( curIndex > 0 ) // do not remove the blank first item
          {
            model.removeElementAt( curIndex );
            setSelectedItem( "" ); // pick the first (blank) item
          }
      }
    else // a non-blank item
      {
        boolean exists = false;
        final int size = model.getSize();
        for( int i=0; i<size; i++ )
          {
            if( item.equals(model.getElementAt(i)) )
              {
                if( PROMOTE_SELECTION )
                  {
                    if( i > 1 )
                      {
                        // Existing entry, move it to the top-spot
                        model.removeElementAt( i );
                        model.insertElementAt( item, curIndex=1 );
                        setSelectedItem( item );
                      }
                  }
                else
                  {
                    curIndex = i;
                  }
                return;
              }
          }
        // It's a new entry
        model.insertElementAt( item, curIndex=1 );
        setSelectedItem( item );
      }
  }

  public class Renderer
    extends JLabel
    implements ListCellRenderer<String>
  {
    public Renderer()
      {
        setOpaque( true );
      }
    public Component getListCellRendererComponent( final JList<? extends String> list,
                                                   final String value,
                                                   final int index,
                                                   final boolean isSelected,
                                                   final boolean hasFocus )
    {
      // NOTE: The JComboBox appears not to honor our colors and border... :-/
      setText( value.toString() );
      if( isSelected )
        {
          list.setForeground( list.getSelectionForeground() );
          list.setBackground( list.getSelectionBackground() );
        }
      else
        {
          list.setForeground( Color.black );
          list.setBackground( index > Settings.get("directory-count-to-remember",10)
                              ? warnColor
                              : happyColor );
        }
      setBorder( hasFocus
                 ? border1
                 : border0 );
      return this;
    }
  }

  private boolean pickingFromList;
  private int curIndex;
  private final MutableComboBoxModel<String> model;
  private static final boolean PROMOTE_SELECTION = true;
  private static Color happyColor = new Color(168,232,198);
  private static Color warnColor  = new Color(222,168,168);
  private final Border border0 = BorderFactory.createEmptyBorder(1,1,1,1);
  private final Border border1 = BorderFactory.createLineBorder(Color.black,1);
}
