package com.ringlord.odfind;

import java.awt.Font;

import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.JLabel;
import javax.swing.GroupLayout;
import javax.swing.GroupLayout.ParallelGroup;
import javax.swing.GroupLayout.SequentialGroup;
import javax.swing.GroupLayout.Alignment;

/**
 * A panel with a two-column multi-row layout that makes for decent
 * looking input forms.
 *
 * @author K. Udo Schuermann
 **/
class Form
  extends JPanel
{
  Form()
  {
    super();

    this.groupLayout = new GroupLayout( this );
    this.groupLayout.setAutoCreateGaps( true );
    this.groupLayout.setAutoCreateContainerGaps( true );
    setLayout( this.groupLayout );

    this.labelColumn = groupLayout.createParallelGroup();
    this.fieldColumn = groupLayout.createParallelGroup();
    this.trailColumn = groupLayout.createParallelGroup();

    final SequentialGroup hGroup = groupLayout.createSequentialGroup();
    hGroup.addGroup( this.labelColumn );
    hGroup.addGroup( this.fieldColumn );
    hGroup.addGroup( this.trailColumn );
    this.groupLayout.setHorizontalGroup( hGroup );

    this.vGroup = groupLayout.createSequentialGroup();
    groupLayout.setVerticalGroup( this.vGroup );
  }

  void add( final JComponent field )
  {
    add( (JComponent)null, field );
  }
  void add( final JComponent label,
            final JComponent field )
  {
    add( label, field, null );
  }
  void add( JComponent label,
            JComponent field,
            JComponent trail )
  {
    if( label == null )
      {
        this.labelColumn.addComponent( label=new JLabel("") );
      }
    else if( label instanceof JLabel )
      {
        this.labelColumn.addComponent( (JLabel)label,
                                       (((JLabel)label).getHorizontalAlignment() == JLabel.TRAILING
                                        ? Alignment.TRAILING
                                        : Alignment.LEADING) );
      }
    else
      {
        this.labelColumn.addComponent( label );
      }
    // ----------------------------------------------------------------------
    if( field == null )
      {
        this.fieldColumn.addComponent( field=new JLabel("") );
      }
    else
      {
        this.fieldColumn.addComponent( field );
      }
    // ----------------------------------------------------------------------
    if( trail == null )
      {
        this.trailColumn.addComponent( trail=new JLabel("") );
      }
    else
      {
        this.trailColumn.addComponent( trail );
      }

    final ParallelGroup row = this.groupLayout.createParallelGroup( Alignment.BASELINE );
    row.addComponent( label ).addComponent( field ).addComponent( trail );
    this.vGroup.addGroup( row );
  }

  private final GroupLayout groupLayout;
  private final SequentialGroup vGroup;
  private final ParallelGroup labelColumn;
  private final ParallelGroup fieldColumn;
  private final ParallelGroup trailColumn;
}
