package com.ringlord.odfind;

import java.awt.Component;
import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Rectangle;
import java.awt.Image;
import java.awt.Window;

import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import java.util.List;
import java.util.ArrayList;

import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.UIManager;
import javax.swing.ImageIcon;
import javax.swing.WindowConstants;

/**
 * <p>The Graphic User Interface, invoked by Main.</p>
 *
 * @author K Udo Schuermann
**/
public class GUI
  implements Runnable
{
  // ==============================
  // Runnable
  // ==============================
  public void run()
  {
    try
      {
        // Try to use the Nimbus look-and-feel
        UIManager.setLookAndFeel( "javax.swing.plaf.nimbus.NimbusLookAndFeel" );
      }
    catch( final Exception e )
      {
        try
          {
            // Use the platform look-and-feel
            UIManager.setLookAndFeel( UIManager.getSystemLookAndFeelClassName() );
          }
        catch( final Exception e2 )
          {
          }
      }

    this.frame = new JFrame( "ODFind" );
    this.frame.setJMenuBar( new MenuBar(this) );

    GUI.setIcons( this.frame );

    final ResultsPanel results = new ResultsPanel( this );
    this.options = new SearchEditor( this, results );

    final JPanel p = new JPanel( new BorderLayout() );
    this.frame.setContentPane( p );
    p.add( options, BorderLayout.NORTH );
    // capture the minimum window size (only the options pane) without
    // the 'result' panel (which adds just a little space at the
    // bottom of the window, space that we don't want). From this we
    // use the saved width (ignoring the height) to construct the
    // initial window. In effect, we will have remembered the width,
    // but don't restore the height from what we remembered.
    this.frame.pack();
    final Rectangle minimumSize = this.frame.getBounds();
    p.add( results, BorderLayout.CENTER );

    final Rectangle mini = Settings.get("main-window-mini-xywh",(Rectangle)null);
    final Rectangle full = Settings.get("main-window-full-xywh",(Rectangle)null);

    if( mini != null )
      {
        // restore the width from what we remembered, but use the
        // height from what the packed window reports (i.e. ignore the
        // remembered height).
        this.miniSize = new Dimension( mini.width, minimumSize.height );
        mini.height = minimumSize.height;
        this.frame.setBounds( mini );
      }
    if( full != null )
      {
        this.fullSize = new Dimension( full.width, full.height );
      }

    this.frame.setDefaultCloseOperation( WindowConstants.DO_NOTHING_ON_CLOSE );
    this.frame.addWindowListener( new WindowAdapter()
      {
        public void windowClosing( final WindowEvent e )
          {
            shutdown();
          }
      } );

    this.frame.setVisible( true );
  }

  public Window window()
  {
    return frame;
  }

  public Component main()
  {
    return frame.getContentPane();
  }

  public void shutdown()
  {
    // @@@ we might want to offer the user the chance to veto this
    // action, just in case that there are searches still running

    this.frame.setVisible( false );

    PasswordManager.shutdown();
    options.shutdown();

    final Rectangle b = frame.getBounds();
    if( isSizedForResults )
      {
        Settings.put( "main-window-full-xywh", b );
      }
    else
      {
        Settings.put( "main-window-mini-xywh", b );
      }
    System.exit( 0 );
  }

  static void setIcons( final JFrame f )
  {
    final List<Image> icons = new ArrayList<Image>();
    ImageIcon wIcon;
    if( (wIcon = Icons.get("appicon")) != null ) { icons.add(wIcon.getImage()); }
    // We could add larger and smaller icons to eliminate generic scaling
    f.setIconImages( icons );
  }

  public void setSizedForResults( final boolean isSizedForResults )
  {
    if( this.isSizedForResults != isSizedForResults )
      {
        if( this.isSizedForResults )
          {
            // capture the current (full) size
            final Rectangle b = frame.getBounds();
            fullSize = new Dimension( b.width, b.height );
            // @@@ save the fullSize
            Settings.put( "main-window-full-xywh", b );

            // change size to miniSize again (which we know is not null
            frame.setSize( miniSize );
          }
        else
          {
            // capture the current (mini) size
            final Rectangle b = frame.getBounds();
            miniSize = new Dimension( b.width, b.height );
            // @@@ save the miniSize
            Settings.put( "main-window-mini-xywh", b );

            // if we have not yet expanded the window to the full size
            // (but: full != fullscreen) then compute a reasonable one
            // for that window, otherwise use the previous size
            if( fullSize == null )
              {
                fullSize = new Dimension( b.width, b.height * 2 );
              }
            fullSize.width = Math.max( fullSize.width, miniSize.width );
            // change size to fullSize
            frame.setSize( fullSize );
          }
        this.isSizedForResults = isSizedForResults;
      }
  }

  private JFrame frame;
  private boolean isSizedForResults;
  private Dimension miniSize;
  private Dimension fullSize;
  private SearchEditor options;
}
