package com.ringlord.odfind;

import java.io.InputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;

import java.net.URL;

import java.util.Map;
import java.util.HashMap;

import javax.swing.ImageIcon;

/**
 * <p>An {@link ImageIcon} cache. Images are referenced merely by
 * their base name, with the extension and the path unspecified: the
 * path is hard-coded within this class as "img/") and the extensions
 * will be tried as ".png", ".jpeg", or ".gif" (in that order).</p>
 *
 * @author K. Udo Schuermann
 **/
public class Icons
{
  /**
   * @param name The name of the ImageIcon sans path or extension.
   * Example, an icon named "duke" will be loaded from a file named
   * "img/duke.png" first, then "img/duke.jpeg", then "img/duke.gif";
   * if none of these match, the icon will be considered missing and
   * produces a null return value.
   *
   * @return <code>null</code> if the icon does not exist under the
   * "img/" directory
   **/
  public static synchronized ImageIcon get( final String name )
  {
    ImageIcon icon = icons.get( name );
    if( icon == null )
      {
        if( (icon = load("img/"+name+".png")) != null )
          {
            icons.put( name, icon );
          }
        else if( (icon = load("img/"+name+".jpeg")) != null )
          {
            icons.put( name, icon );
          }
        else if( (icon = load("img/"+name+".gif")) != null )
          {
            icons.put( name, icon );
          }
      }
    return icon;
  }

  private static ImageIcon load( final String filename )
  {
    InputStream f = null;
    try
      {
        f = Icons.class.getClassLoader().getResourceAsStream( filename );
        if( f != null )
          {
            final ByteArrayOutputStream b = new ByteArrayOutputStream();
            int n;
            while( (n = f.read(buf)) > -1 )
              {
                b.write( buf, 0, n );
              }
            return new ImageIcon( b.toByteArray() );
          }
        return null;
      }
    catch( IOException x )
      {
        return null;
      }
    finally
      {
        if( f != null )
          {
            try
              {
                f.close();
              }
            catch( IOException x )
              {
                // Can't clean up, then leave the dirt for the garbage
                // collector to sweep up...
              }
          }
      }
  }

  private static final Map<String,ImageIcon> icons = new HashMap<String,ImageIcon>();
  private static final byte[] buf = new byte[ 256 ];
}
