package com.ringlord.odfind;

import java.awt.Desktop;
import java.awt.Image;
import java.awt.Font;
import java.awt.Color;
import java.awt.Rectangle;
import java.awt.RenderingHints;
import java.awt.Graphics2D;
import java.awt.image.BufferedImage;

import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import java.io.IOException;

import java.net.URI;
import java.net.URL;
import java.net.URISyntaxException;

import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenuBar;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.KeyStroke;
import javax.swing.ImageIcon;
import javax.swing.SwingWorker;
import javax.swing.WindowConstants;
import javax.swing.JDialog;

import static java.awt.Dialog.ModalityType.APPLICATION_MODAL;

import static com.ringlord.odfind.Main.VERSION;
import static com.ringlord.odfind.Main.VERDATE;

public class MenuBar
  extends JMenuBar
  implements ActionListener
{
  public MenuBar( final GUI gui )
  {
    super();
    this.gui = gui;

    menuNew = new JMenuItem( "New" );
    menuOpen = new JMenuItem( "Open \u2026" );
    menuSave = new JMenuItem( "Save \u2026" );
    menuQuit = new JMenuItem( "Exit" );

    setupMenuItem( menuNew, KeyEvent.VK_N, KeyStroke.getKeyStroke("ctrl N") );
    setupMenuItem( menuOpen, KeyEvent.VK_O, KeyStroke.getKeyStroke("ctrl O") );
    setupMenuItem( menuSave, KeyEvent.VK_E, KeyStroke.getKeyStroke("ctrl S") );
    setupMenuItem( menuQuit, KeyEvent.VK_Q, KeyStroke.getKeyStroke("ctrl Q") );

    menuNew.addActionListener( this );
    menuOpen.addActionListener( this );
    menuSave.addActionListener( this );
    menuQuit.addActionListener( this );

    menuNew.setToolTipText( "<html>"+
                            "Reset search fields to their<br>"+
                            "defaults to start a new search." );
    menuOpen.setToolTipText( "Open a previously saved search." );
    menuSave.setToolTipText( "<html>"+
                             "Save the current search, the<br>"+
                             "criteria and available results,<br>"+
                             "for later restoration." );
    menuQuit.setToolTipText( "Exit/Quit ODFind." );

    // ----------------------------------------------------------------------

    menuEditDocument = new JMenuItem( "Document \u2026" );
    menuSettings = new JMenuItem( "Settings \u2026" );

    menuEditDocument.addActionListener( this );
    menuSettings.addActionListener( this );

    menuEditDocument.setToolTipText( "<html>"+
                                     "Open the document selected<br>"+
                                     "in the search result list." );
    menuSettings.setToolTipText( "<html>"+
                                 "Modify persistent settings." );

    // ----------------------------------------------------------------------

    menuPwdManager = new JMenuItem( "Password manager \u2026" );
    menuPassword = new JMenuItem( "Password \u2026" );

    menuPwdManager.addActionListener( this );
    menuPassword.addActionListener( this );

    menuPwdManager.setToolTipText( "<html>"+
                                   "When encrypted documents are<br>"+
                                   "encountered during a search,<br>"+
                                   "the password manager handles<br>"+
                                   "the interactive collection of<br>"+
                                   "passwords, and re-attempts to<br>"+
                                   "access these documents." );
    menuPassword.setToolTipText( "<html>"+
                                 "View the password of the encrypted<br>"+
                                 "document selected in the search result<br>"+
                                 "listing." );

    // ----------------------------------------------------------------------

    menuOnline = new JMenuItem( "Online help \u2026" );
    menuAbout = new JMenuItem( "About ODFind \u2026" );

    menuOnline.addActionListener( this );
    menuAbout.addActionListener( this );

    menuOnline.setToolTipText( "<html>"+
                               "Invoke a web browser to visit<br>"+
                               "http://ringlord.com/odfind/" );
    menuAbout.setToolTipText( "<html>"+
                              "Show basic information<br>"+
                              "about ODFind, including<br>"+
                              "version and release date." );

    setupMenuItem( menuOnline, KeyEvent.VK_F1, KeyStroke.getKeyStroke("F1") );

    // ----------------------------------------------------------------------

    final JMenu fileMenu = new JMenu( "File" );
    final JMenu editMenu = new JMenu( "Edit" );
    final JMenu viewMenu = new JMenu( "View" );
    final JMenu helpMenu = new JMenu( "Help" );

    fileMenu.setMnemonic( 'F' );
    editMenu.setMnemonic( 'E' );
    viewMenu.setMnemonic( 'V' );
    helpMenu.setMnemonic( 'H' );

    // ----------------------------------------------------------------------

    fileMenu.add( menuNew );
    fileMenu.add( menuOpen );
    fileMenu.add( menuSave );
    fileMenu.add( menuQuit );

    editMenu.add( menuEditDocument );
    editMenu.add( menuSettings );

    viewMenu.add( menuPwdManager );
    // @@@    viewMenu.add( menuPassword );

    helpMenu.add( menuOnline );
    helpMenu.add( menuAbout );

    add( fileMenu );
    add( editMenu );
    add( viewMenu );
    add( helpMenu );

    menuNew.setEnabled( false );
    menuOpen.setEnabled( false );
    menuSave.setEnabled( false );
    menuEditDocument.setEnabled( false );
    menuPassword.setEnabled( false );
  }

  private void setupMenuItem( final JMenuItem menuItem,
                              final Integer mnemonic,
                              final KeyStroke accelerator )
  {
    if( mnemonic != null )
      {
        menuItem.setMnemonic( mnemonic.intValue() );
      }
    if( accelerator != null )
      {
        menuItem.setAccelerator( accelerator );
      }
  }

  // ======================================================================
  // ActionListener
  // ======================================================================
  public void actionPerformed( final ActionEvent e )
  {
    final Object o = e.getSource();
    if( o == menuNew )
      {
      }
    else if( o == menuOpen )
      {
      }
    else if( o == menuSave )
      {
      }
    else if( o == menuQuit )
      {
        gui.shutdown();
      }
    // ----------------------------------------------------------------------
    else if( o == menuSettings )
      {
        final JDialog d = new JDialog( gui.window(),
                                       "Default Settings",
                                       APPLICATION_MODAL );
        d.setContentPane( new OptionsPanel(d) );
        // NOTE: Calling pack() TWICE allows the JTextArea
        // to get its act together, and when we call pack
        // the second time, the layout has become stable:
        d.pack(); // first (layout not yet "good")
        d.pack(); // second (all should be well now!)
        final Rectangle r1 = d.getBounds();

        final Rectangle r2 = Settings.get("settings-window-xywh",(Rectangle)null);
        if( r2 != null )
          {
            // From previously saved dimensions use the larger width
            // and height, but keep the x,y location
            if( r2.width > r1.width )
              {
                r1.width = r2.width;
              }
            if( r2.height > r1.height )
              {
                r1.height = r2.height;
              }
            r1.x = r2.x;
            r1.y = r2.y;
          }
        else
          {
            // No previous dimensions, so center the window
            final Rectangle r3 = gui.window().getBounds();
            final int centerX = r3.x + (r3.width / 2);
            final int centerY = r3.y + (r3.height / 2);
            r1.x = centerX - (r1.width / 2);
            r1.y = centerY - (r1.width / 2);
          }
        d.setBounds( r1 );
        d.setVisible( true );

        final Rectangle r4 = d.getBounds();
        Settings.put("settings-window-xywh",r4);
      }
    // ----------------------------------------------------------------------
    else if( o == menuPwdManager )
      {
        PasswordManager.getInstance().setVisible( true );
      }
    // ----------------------------------------------------------------------
    else if( o == menuPassword )
      {
        // @@@ view the currently selected document's password
      }
    // ----------------------------------------------------------------------
    else if( o == menuOnline )
      {
        final Desktop desktop = Desktop.getDesktop();
        new SwingWorker<Void,Void>()
          {
            public Void doInBackground()
            {
              try
                {
                  System.err.println( "Asking your web browser to visit "+
                                      "http://ringlord.com/odfind/" );
                  menuOnline.setEnabled( false );
                  desktop.browse( new URI("http://ringlord.com/odfind/") );
                  System.err.println( "Finished passing request to your web browser" );
                  // re-enable the menu entry if the operation did not fail
                  menuOnline.setEnabled( true );
                }
              catch( final URISyntaxException x )
                {
                }
              catch( final IOException x )
                {
                }
              return null;
            }
          }.execute();
      }
    else if( o == menuAbout )
      {
        if( about == null )
          {
            about = new JFrame( "About ODFind" );
            about.setAlwaysOnTop( true );
            about.setResizable( false );
            GUI.setIcons( about );

            final ClassLoader cl = GUI.class.getClassLoader();
            final URL url = cl.getResource( "img/about.png" );
            if( url != null )
              {
                final ImageIcon splash = new ImageIcon(about.getToolkit().createImage(url) );
                final Image img = new BufferedImage( splash.getIconWidth(),
                                                     splash.getIconHeight(),
                                                     BufferedImage.TYPE_INT_RGB );
                final Graphics2D g2 = (Graphics2D)img.getGraphics();
                g2.setRenderingHint( RenderingHints.KEY_ANTIALIASING,
                                     RenderingHints.VALUE_ANTIALIAS_ON );
                g2.setRenderingHint( RenderingHints.KEY_INTERPOLATION,
                                     RenderingHints.VALUE_INTERPOLATION_BICUBIC );
                g2.drawImage( splash.getImage(), 0, 0, null );
                final int x = 8;

                g2.setFont( new Font("SansSerif",Font.BOLD,15) );
                g2.setColor( Color.black );
                g2.drawString( "v "+VERSION, x+2, 156+1 );
                g2.setColor( Color.white );
                g2.drawString( "v "+VERSION, x+1, 156 );

                g2.setFont( new Font("SansSerif",Font.BOLD,9) );
                g2.setColor( Color.black );
                g2.drawString( VERDATE, x+1, 170+1 );
                g2.setColor( Color.white );
                g2.drawString( VERDATE, x, 170 );

                g2.dispose();

                final JLabel l = new JLabel( new ImageIcon(img) );
                l.setBorder( null );
                about.getContentPane().add( l );

                l.addMouseListener( new MouseAdapter()
                  {
                    public void mouseClicked( final MouseEvent e )
                    {
                      about.setVisible( false );
                    }
                  } );
                about.getContentPane().add( l );
              }

            about.addKeyListener( new KeyAdapter()
              {
                public void keyPressed( final KeyEvent e )
                {
                  about.setVisible( false );
                }
                public void keyTyped( final KeyEvent e )
                {
                  about.setVisible( false );
                }
              } );

            about.pack();

            about.setDefaultCloseOperation( WindowConstants.DO_NOTHING_ON_CLOSE );
            about.addWindowListener( new WindowAdapter()
              {
                public void windowClosing( final WindowEvent e )
                {
                  about.setVisible( false );
                }
                public void windowDeactivated( final WindowEvent e )
                {
                  about.setVisible( false );
                }
                public void windowIconified( final WindowEvent e )
                {
                  about.setVisible( false );
                }
              } );
          }

        if( ! about.isVisible() )
          {
            about.setLocationRelativeTo( gui.main() );
            about.setVisible( true );
            final int extendedState = about.getExtendedState();
            if( (extendedState & JFrame.ICONIFIED) != 0 )
              {
                about.setExtendedState( extendedState ^ JFrame.ICONIFIED );
              }
            about.toFront();
          }
      }
  }

  private JFrame about;
  private final GUI gui;
  //
  private final JMenuItem menuNew, menuOpen, menuSave, menuQuit;
  private final JMenuItem menuEditDocument, menuSettings;
  private final JMenuItem menuPwdManager, menuPassword;
  private final JMenuItem menuOnline, menuAbout;
}