package com.ringlord.odfind;

import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.GridLayout;

import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;

import javax.swing.JPanel;
import javax.swing.JLabel;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JTextArea;
import javax.swing.JFormattedTextField;
import javax.swing.JDialog;
import javax.swing.JComponent;
import javax.swing.KeyStroke;

public class OptionsPanel
  extends JPanel
{
  OptionsPanel( final JDialog dialog )
  {
    super( new BorderLayout() );

    final JTextArea intro = new JTextArea();
    intro.setEditable( false );
    intro.setLineWrap( true );
    intro.setWrapStyleWord( true );
    intro.setText( "These options are applied when ODFind starts up "+
                   "the next time. None of these options affect the "+
                   "command line version of ODFind." );
    intro.setOpaque( false );

    final Form form = new Form();

    final JCheckBox optCaseSensitive =
      new JCheckBox("Case sensitive",
                    Settings.get("search-case-sensitive",false));
    final JCheckBox optRecursive =
      new JCheckBox("Recursive search",
                    Settings.get("search-recursive",true));
    final JCheckBox optFollowSymlinks =
      new JCheckBox("Follow links",
                    Settings.get("search-follow-symlinks",false));
    final JCheckBox optSearchRawXML =
      new JCheckBox("Search raw XML",
                    Settings.get("search-raw-xml",false));
    final JCheckBox optSearchAllParts =
      new JCheckBox("Search all parts",
                    Settings.get("search-all-parts",false));
    final JFormattedTextField optRememberDirs =
      new JFormattedTextField(new Integer(Settings.get("directory-count-to-remember",10)));

    final JLabel rememberLabel;

    form.add( optCaseSensitive );
    form.add( optRecursive );
    form.add( optFollowSymlinks );
    form.add( optSearchRawXML );
    form.add( optSearchAllParts );
    form.add( rememberLabel=new JLabel("Remember",JLabel.TRAILING),
              optRememberDirs,
              new JLabel("directories") );

    final JButton accept, cancel;
    final JPanel buttons = new JPanel( new GridLayout(1,0,2,0) );
    buttons.add( accept=new JButton("Save",Icons.get("accept")) );
    buttons.add( cancel=new JButton("Cancel",Icons.get("cancel")) );
    final JPanel cmds = new JPanel( new FlowLayout(FlowLayout.RIGHT) );
    cmds.add( buttons );

    add( intro, BorderLayout.NORTH );
    add( form,  BorderLayout.CENTER );
    add( cmds,  BorderLayout.SOUTH );

    optCaseSensitive.setMnemonic(KeyEvent.VK_C);
    optRecursive.setMnemonic(KeyEvent.VK_R);
    optFollowSymlinks.setMnemonic(KeyEvent.VK_F);
    optSearchRawXML.setMnemonic(KeyEvent.VK_W);
    optSearchAllParts.setMnemonic(KeyEvent.VK_A);

    rememberLabel.setDisplayedMnemonic(KeyEvent.VK_M);
    rememberLabel.setLabelFor(optRememberDirs);

    accept.setMnemonic(KeyEvent.VK_S);  // works fine
    cancel.setMnemonic(KeyEvent.VK_N);          // doesn't seem to work
    cancel.setMnemonic(KeyEvent.VK_ESCAPE);     // doesn't seem to work

    accept.addActionListener( new ActionListener()
      {
        public void actionPerformed( final ActionEvent e )
        {
          Settings.put( "search-case-sensitive",
                        optCaseSensitive.isSelected() );
          Settings.put( "search-recursive",
                        optRecursive.isSelected() );
          Settings.put( "search-follow-symlinks",
                        optFollowSymlinks.isSelected() );
          Settings.put( "search-raw-xml",
                        optSearchRawXML.isSelected() );
          Settings.put( "search-all-parts",
                        optSearchAllParts.isSelected() );
          try
            {
              final int remember = Integer.parseInt( optRememberDirs.getText() );
              Settings.put( "directory-count-to-remember",
                            Math.max(0,remember) );
            }
          catch( Exception x )
            {
              // we could complain, but what can we do about it?
            }
          dialog.setVisible( false );
        }
      } );
    ActionListener cancelListener;
    cancel.addActionListener( cancelListener=new ActionListener()
      {
        public void actionPerformed( final ActionEvent e )
        {
          dialog.setVisible( false );
        }
      } );
    // the ESCAPE key will also invoke th ActionListener attached to
    // the cancel button
    registerKeyboardAction( cancelListener,
                            KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0),
                            JComponent.WHEN_IN_FOCUSED_WINDOW );
  }
}
