package com.ringlord.odfind;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Rectangle;

import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import java.io.Console;
import java.io.IOException;

import java.nio.file.Path;

import java.util.List;
import java.util.ArrayList;

import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JLabel;
import javax.swing.JButton;
import javax.swing.JTextArea;
import javax.swing.JPasswordField;
import javax.swing.SwingWorker;
import javax.swing.BorderFactory;
import javax.swing.WindowConstants;

import com.ringlord.odf.Container;
import com.ringlord.odf.Entry;

public class PasswordManager
{
  public static synchronized PasswordManager getInstance()
  {
    if( instance == null )
      {
        instance = new PasswordManager();
      }
    return instance;
  }

  private PasswordManager()
  {
    super();
  }

  public void setVisible( final boolean isVisible )
  {
    if( this.frame == null )
      {
        this.frame = new JFrame( "ODFind Password Manager" );
        this.frame.setDefaultCloseOperation( WindowConstants.DO_NOTHING_ON_CLOSE );
        this.frame.addWindowListener( new WindowAdapter()
          {
            public void windowClosing( final WindowEvent e )
            {
              shutdown();
            }
          } );
        GUI.setIcons( this.frame );

        final JTextArea intro = new JTextArea();
        intro.setEditable( false );
        intro.setLineWrap( true );
        intro.setWrapStyleWord( true );
        intro.setText( "Provide ODFind with passwords to unlock encrypted "+
                       "documents. All passwords that you enter here are "+
                       "remembered only until ODFind exits. Each password is "+
                       "tried against all encrypted documents to save you "+
                       "from having to type them again." );
        intro.setOpaque( false );

        final JLabel label = new JLabel( "Password:", JLabel.TRAILING );
        this.password = new JPasswordField( 8 );
        final JButton clearButton = new JButton( Icons.get("clear") );

        final JPanel inputs = new JPanel( new BorderLayout() );
        inputs.add( label,       BorderLayout.WEST );
        inputs.add( password,    BorderLayout.CENTER );
        inputs.add( clearButton, BorderLayout.EAST );
        inputs.setBorder( BorderFactory.createEmptyBorder(8,0,8,0) );

        final JPanel top = new JPanel( new BorderLayout() );
        top.add( intro,  BorderLayout.CENTER );
        top.add( inputs, BorderLayout.SOUTH );

        this.info = new JLabel("");
        this.info.setOpaque( true );
        this.info.setBorder( BorderFactory.createEmptyBorder(0,2,0,0) );
        updateInfo( null );

        final JPanel p = new JPanel( new BorderLayout() );
        p.add( top,  BorderLayout.CENTER );
        p.add( info, BorderLayout.SOUTH );
        p.setBorder( BorderFactory.createEmptyBorder(8,8,8,8) );

        clearButton.setToolTipText( "No passwords" );
        clearButton.setEnabled( false );
        clearButton.addActionListener( new ActionListener()
          {
            public void actionPerformed( final ActionEvent e )
            {
              passwords.clear();
              clearButton.setEnabled( false );
              clearButton.setToolTipText( "No passwords" );
            }
          } );

        password.addActionListener( new ActionListener()
          {
            public void actionPerformed( final ActionEvent e )
            {
              final String p = new String( password.getPassword() );
              password.setText( "" );
              synchronized( passwords )
                {
                  if( ! passwords.contains(p) )
                    {
                      passwords.add( p );
                      clearButton.setEnabled( true );
                      clearButton.setToolTipText( "Forget "+passwords.size()+" password"+
                                                  (passwords.size()==1?"":"s") );
                    }
                }
              synchronized( waiting )
                {
                  List<Item> done = null;
                  for( Item item : waiting )
                    {
                      final Path file = item.getFile();
                      final Result result = data( file );
                      if( result != null )
                        {
                          // Decrypt worked!
                          item.getInspector().search( result.data, file, result.password );
                          info.setBackground( happyColor );
                          if( done == null )
                            {
                              done = new ArrayList<Item>();
                            }
                          done.add( item );
                        }
                    }
                  if( done != null )
                    {
                      for( Item item : done )
                        {
                          waiting.remove( item );
                          item.getResultComponent().refreshWaiting();
                        }
                    }
                  updateInfo( null );
                }
            }
          } );

        this.frame.setContentPane( p );
        final Rectangle dims = Settings.get("pwds-window-xywh",(Rectangle)null);
        if( dims == null )
          {
            this.frame.pack();
            this.frame.setSize( new java.awt.Dimension(370,195) );
          }
        else
          {
            this.frame.setBounds( dims );
          }
      }
    if( isVisible )
      {
        this.frame.setVisible( true );
        password.requestFocusInWindow();
      }
    else
      {
        this.frame.setVisible( false );
      }
  }

  public static void shutdown()
  {
    if( instance != null )
      {
        instance.setVisible( false );
        Settings.put( "pwds-window-xywh", instance.frame.getBounds() );
      }
  }

  public int getWaitingCount()
  {
    return this.waiting.size();
  }

  private void updateInfo( final Boolean isHappy )
  {
    final int count = this.waiting.size();
    info.setText( count+" encrypted file"+
                  (count==1?"":"s")+" awaiting "+
                  "a correct password" );
    if( isHappy == null )
      {
        info.setBackground( count == 0
                            ? happyColor
                            : warnColor );
      }
    else
      {
        info.setBackground( isHappy
                            ? happyColor
                            : warnColor );
      }
  }

  void setCLI( final boolean checkEncryptedFiles,
               final boolean interactiveConsole )
  {
    this.cliMode = true;
    this.checkEncryptedFiles = checkEncryptedFiles;
    if( interactiveConsole )
      {
        if( (this.console = System.console()) == null )
          {
            System.err.println( "No Console device available" );
          }
      }
    else
      {
        this.console = null;
      }
  }

  int getCollectedCount()
  {
    return collection.size();
  }

  void finishCLI()
  {
    for( Item item : collection )
      {
        final Path file = item.getFile();
        while( true )
          {
            final Result result = data( file );
            if( result == null )
              {
                if( console != null )
                  {
                    System.err.println( "Encrypted file: "+file );
                    final char[] pwd = System.console().readPassword( "%s", "Enter Password: ");
                    final String password = new String( pwd );
                    if( password.length() == 0 )
                      {
                        break;
                      }
                    add( password );
                    continue;
                  }
                else
                  {
                    break;
                  }
              }
            // Decrypt worked!
            item.getInspector().search( result.data, file, result.password );
            break;
          }
      }
  }

  /**
   * Add a password without typing it into the GUI (i.e. this is for
   * CLI usage)
   **/
  boolean add( final String password )
  {
    if( this.password == null )
      {
        if( ! passwords.contains(password) )
          {
            passwords.add( password );
            return true;
          }
        return false;
      }
    else
      {
        // The ONLY reason we throw this exception is because the
        // anonymous ActionListenerS handle the enabling/disabling of
        // the relevant GUI elements, such as the clearButton, and we
        // have no clean way of doing that here until we make the
        // clearButton more widely available to this class... Don't
        // really see a need for this as there should be no reason
        // to add passwords behind the scenes after the GUI has been
        // created...
        throw new IllegalStateException( "Cannot add passwords to "+
                                         "PasswordManager after "+
                                         "the GUI was built." );
      }
  }

  void add( final TreeWalker.Inspector inspector,
            final ResultsPanel.ResultComponent r,
            final Path file,
            final Container odf,
            final Entry content )
  {
    if( cliMode )
      {
        if( checkEncryptedFiles )
          {
            collection.add( new Item(inspector,null,file) );
          }
        try
          {
            odf.close();
          }
        catch( IOException x )
          {
          }
      }
    else
      {
        // Quick, while the file/container is open, check all collected
        // passwords. But before we interact with the user, close the
        // file/container so that we don't deplete the system of open file
        // handles...
        new SwingWorker<Void,Void>()
          {
            public Void doInBackground()
            {
              try
                {
                  for( String pwd : passwords )
                    {
                      try
                        {
                          final byte[] data = content.data( pwd );
                          inspector.search( data, file, pwd );
                          return null;
                        }
                      // A fairly large variety of exceptions can occur,
                      // all of which essentially mean, "sorry, no data
                      // for you, my friend!"
                      catch( Exception x )
                        {
                        }
                    }
                }
              finally
                {
                  try
                    {
                      odf.close();
                    }
                  catch( IOException x )
                    {
                    }
                }

              synchronized( waiting )
                {
                  waiting.add( new Item(inspector,r,file) );
                  r.refreshWaiting();
                  updateInfo( false );
                }
              instance.frame.setVisible( true );
              return null;
            }
          }.execute();
      }
  }

  /**
   * Obtains the contents of the given ODF file, applying all
   * collected passwords if it is encrypted.
   *
   * @return null The file is not an ODF container, contains no
   * "content.xml", or it is encrypted but none of the collected
   * passwords could decrypt it.
   **/
  Result data( final Path file )
  {
    try
      {
        final Container odf = new Container( file.toFile() );
        try
          {
            final Entry content = odf.get( "content.xml" );
            if( content == null )
              {
                return null;
              }
            if( content.isEncrypted() )
              {
                for( String pwd : passwords )
                  {
                    try
                      {
                        final byte[] data = content.data( pwd );
                        return new Result(data,pwd);
                      }
                    catch( Exception x )
                      {
                        // not the right password
                      }
                  }
              }
            else
              {
                return new Result(content.data(),null);
              }
          }
        finally
          {
            try
              {
                odf.close();
              }
            catch( Exception x )
              {
              }
          }
      }
    catch( Exception x )
      {
      }
    return null;
  }

  private class Result
  {
    Result( final byte[] data,
            final String password )
    {
      super();
      this.data = data;
      this.password = password;
    }
    final byte[] data;
    final String password;
  }

  private class Item
  {
    Item( final TreeWalker.Inspector inspector,
          final ResultsPanel.ResultComponent r,
          final Path file )
    {
      super();
      this.inspector = inspector;
      this.resultComponent = r;
      this.file = file;
    }
    public TreeWalker.Inspector getInspector()
    {
      return inspector;
    }
    public ResultsPanel.ResultComponent getResultComponent()
    {
      return resultComponent;
    }
    public Path getFile()
    {
      return file;
    }
    private final TreeWalker.Inspector inspector;
    private final ResultsPanel.ResultComponent resultComponent;
    private final Path file;
  }

  //
  private JFrame frame;
  private JPasswordField password;
  private JLabel info;
  private boolean cliMode;
  private boolean checkEncryptedFiles;  // cliMode only
  private Console console;              // cliMode only
  //
  private final List<Item> collection = new ArrayList<Item>();  // for cli usage
  private final List<Item> waiting = new ArrayList<Item>();     // for gui usage
  private final List<String> passwords = new ArrayList<String>();
  //
  private static PasswordManager instance;
  private static Color happyColor = new Color(168,232,198);
  private static Color warnColor  = new Color(222,168,168);
}
