package com.ringlord.odfind;

import java.awt.Desktop;
import java.awt.Component;
import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.awt.Font;
import java.awt.Color;

import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseEvent;
import java.awt.event.KeyEvent;

import java.io.File;

import java.nio.file.Path;

import java.util.List;
import java.util.ArrayList;

import javax.swing.JPanel;
import javax.swing.JList;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPopupMenu;
import javax.swing.JMenuItem;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.BorderFactory;
import javax.swing.SwingWorker;
import javax.swing.DefaultListModel;
import javax.swing.ListCellRenderer;
import javax.swing.ImageIcon;

import javax.swing.border.Border;

public class ResultsPanel
  extends JPanel
{
  public ResultsPanel( final GUI gui )
  {
    super( new BorderLayout() );
    this.gui = gui;
    this.tabs = new JTabbedPane();
    this.tabs.setVisible( false );
    add( tabs, BorderLayout.CENTER );
    setBorder( BorderFactory.createEmptyBorder(8,8,8,8) );
  }

  public ResultComponent addSearchPanel( final TreeWalker walker,
                                         final String[] queries )
  {
    tabs.setVisible( true );
    gui.setSizedForResults( true );

    final ResultComponent r = new ResultComponent( walker );
    final JButton close = TabTitle.createCloseButton( tabs, r );
    close.setToolTipText( "Close this tab" );
    close.addActionListener( new ActionListener()
      {
        public void actionPerformed( final ActionEvent e )
        {
          final TabTitle.CloseButton close = (TabTitle.CloseButton)e.getSource();
          final JTabbedPane tabs = close.getTabs();
          final ResultComponent r = (ResultComponent)close.getComponent();
          r.done();
          tabs.remove( r );
          if( tabs.getTabCount() == 0 )
            {
              gui.setSizedForResults( false );
            }
        }
      } );

    final StringBuilder sb = new StringBuilder();
    sb.append( "<html>" );
    for( int i=0; i<queries.length; i++ )
      {
        if( i > 0 )
          {
            sb.append( "<br>" );
          }
        sb.append( "<b>"+entityEscape(queries[i])+"</b>" );
      }

    tabs.addTab( null, r );
    final int index = tabs.getTabCount() - 1;
    final TabTitle tabTitle = new TabTitle( queries[0]+(queries.length > 1
                                                        ? " \u2026"
                                                        : ""),
                                            Icons.get("search-result"), close );
    tabs.setTabComponentAt( index, tabTitle );
    tabs.setToolTipTextAt( index, sb.toString() );
    tabs.setSelectedIndex( tabs.getTabCount() - 1 );
    return r;
  }

  class ResultComponent
    extends JPanel
  {
    public ResultComponent( final TreeWalker walker )
    {
      super( new BorderLayout() );

      this.walker = walker;

      final JPanel b = new JPanel( new GridLayout(1,0,2,0) );
      b.add( stopButton  = new JButton("Stop",Icons.get("stop")) );
      b.add( closeButton = new JButton("Close",Icons.get("close")) );

      stopButton.setMnemonic(KeyEvent.VK_T);
      closeButton.setMnemonic(KeyEvent.VK_O);

      final JPanel c = new JPanel( new BorderLayout() );
      c.add( this.feedback = new JLabel(""), BorderLayout.CENTER );
      c.add( b,                              BorderLayout.EAST );
      c.setBorder( BorderFactory.createEmptyBorder(2,0,2,0) );

      this.originalFeedbackFont = this.feedback.getFont();
      this.feedback.setFont( new Font("SansSerif",Font.PLAIN,10) );
      this.feedback.setBorder( BorderFactory.createEmptyBorder(0,4,0,0) );
      this.feedback.addMouseListener( new MouseListener()
        {
          public void mouseEntered( final MouseEvent e )
          {
            if( PasswordManager.getInstance().getWaitingCount() > 0 )
              {
                plainFeedbackBgColor = feedback.getBackground();
                feedback.setOpaque( true );
                feedback.setBackground( hintFeedbackBgColor );
              }
          }
          public void mouseExited( final MouseEvent e )
          {
            if( plainFeedbackBgColor != null )
              {
                feedback.setBackground( plainFeedbackBgColor );
                feedback.setOpaque( false );
                plainFeedbackBgColor = null;
              }
          }
          public void mouseClicked( final MouseEvent e )
          {
            final PasswordManager pwMan = PasswordManager.getInstance();
            if( pwMan.getWaitingCount() > 0 )
              {
                pwMan.setVisible( true );
              }
          }
          public void mousePressed( final MouseEvent e )
          {
          }
          public void mouseReleased( final MouseEvent e )
          {
          }
        } );

      this.info = new JLabel( "0 files matched" );
      this.info.setBorder( BorderFactory.createEmptyBorder(4,6,2,0) );

      this.matchingFiles = new JList<Item>( new DefaultListModel<Item>() );
      this.matchingFiles.setCellRenderer( new ItemRenderer() );
      this.matchingFiles.setBorder( BorderFactory.createEmptyBorder(4,4,4,4) );
      this.matchingFiles.addMouseListener( new MouseListener()
        {
          @Override
          public void mouseEntered( final MouseEvent e )
          {
          }
          @Override
          public void mouseExited( final MouseEvent e )
          {
          }
          @Override
          public void mouseClicked( final MouseEvent e )
          {
            final DefaultListModel<Item> model =
              (DefaultListModel<Item>)matchingFiles.getModel();
            final int index = matchingFiles.locationToIndex( e.getPoint() );
            final Item cur = (index < 0 ? null : model.getElementAt(index));
            if( e.isPopupTrigger() )
              {
                doPopup( e, cur );
              }
            else if( e.getClickCount() == 2 )
              {
                final Path file = cur.getFile();

                final Desktop desktop = Desktop.getDesktop();
                new SwingWorker<Void,Void>()
                  {
                    public Void doInBackground()
                    {
                      try
                        {
                          desktop.open( file.toFile() );
                          // @@@ as this method seems to come back
                          // rather quickly, we can now initiate some
                          // sort of feedback indicator (maybe some
                          // sort of spinner on this file's icon?)
                          // that should probably time out after a few
                          // seconds, by which time the editor should
                          // have finished loading the file.
                        }
                      catch( Exception x )
                        {
                          // @@@ need to give user an indication that this failed
                          System.err.println( "Tried but failed to open the file" );
                          x.printStackTrace( System.err );
                        }
                      return null;
                    }
                  }.execute();
              }
          }
          @Override
          public void mousePressed( final MouseEvent e )
          {
            final DefaultListModel<Item> model =
              (DefaultListModel<Item>)matchingFiles.getModel();
            final int index = matchingFiles.locationToIndex( e.getPoint() );
            final Item cur = (index < 0 ? null : model.getElementAt(index));
            if( e.isPopupTrigger() )
              {
                doPopup( e, cur );
              }
          }
          @Override
          public void mouseReleased( final MouseEvent e )
          {
            final DefaultListModel<Item> model =
              (DefaultListModel<Item>)matchingFiles.getModel();
            final int index = matchingFiles.locationToIndex( e.getPoint() );
            final Item cur = (index < 0 ? null : model.getElementAt(index));
            if( e.isPopupTrigger() )
              {
                doPopup( e, cur );
              }
          }
        } );

      add( info,                                 BorderLayout.NORTH );
      add( new JScrollPane(this.matchingFiles),  BorderLayout.CENTER );
      add( c,                                    BorderLayout.SOUTH ); // @@@ needs to move right

      stopButton.addActionListener( new ActionListener()
        {
          public void actionPerformed( final ActionEvent e )
          {
            stopped = true;
            walker.stop();
            stopButton.setEnabled( false );
          }
        } );
      closeButton.addActionListener( new ActionListener()
        {
          public void actionPerformed( final ActionEvent e )
          {
            stopped = true;
            walker.stop();
            stopButton.setEnabled( false );
            closeButton.setEnabled( false );

            final ResultComponent r = (ResultComponent)tabs.getSelectedComponent();
            r.done();
            tabs.remove( r );
            if( tabs.getTabCount() == 0 )
              {
                gui.setSizedForResults( false );
              }
          }
        } );
    }

    private void doPopup( final MouseEvent e,
                          final Item item )
    {
      final Path file = (item == null ? null : item.getFile());
      final Desktop desktop = Desktop.getDesktop();

      feedback.setText( "" );

      popupTarget = item;
      if( popupMenu == null )
        {
          this.popupMenuOpen  = new JMenuItem( "Open \u2026" );
          this.popupMenuEdit  = new JMenuItem( "Edit \u2026" );
          this.popupMenuPrint = new JMenuItem( "Print \u2026" );
          this.popupMenuPwd   = new JMenuItem( "Show password \u2026" );
          this.popupMenuHide  = new JMenuItem( "Hide" );
          this.popupMenuShow  = new JMenuItem( "Reveal all" );

          this.popupMenuOpen.addActionListener( new ActionListener()
            {
              public void actionPerformed( final ActionEvent e )
              {
                new SwingWorker<Void,Void>()
                  {
                    public Void doInBackground()
                    {
                      try
                        {
                          desktop.open( file.toFile() );
                        }
                      catch( Exception x )
                        {
                          // @@@ need to give user an indication that this failed
                          System.err.println( "Supposedly OPEN is supported "+
                                              "but the operation FAILED!" );
                          x.printStackTrace( System.err );
                        }
                      return null;
                    }
                  }.execute();
              }
            } );
          this.popupMenuEdit.addActionListener( new ActionListener()
            {
              public void actionPerformed( final ActionEvent e )
              {
                new SwingWorker<Void,Void>()
                  {
                    public Void doInBackground()
                    {
                      try
                        {
                          desktop.edit( file.toFile() );
                        }
                      catch( Exception x )
                        {
                          // @@@ need to give user an indication that this failed
                          System.err.println( "Supposedly EDIT is supported "+
                                              "but the operation FAILED!" );
                          x.printStackTrace( System.err );
                        }
                      return null;
                    }
                  }.execute();
              }
            } );
          this.popupMenuPrint.addActionListener( new ActionListener()
            {
              public void actionPerformed( final ActionEvent e )
              {
                new SwingWorker<Void,Void>()
                  {
                    public Void doInBackground()
                    {
                      try
                        {
                          desktop.print( file.toFile() );
                        }
                      catch( Exception x )
                        {
                          // @@@ need to give user an indication that this failed
                          System.err.println( "Supposedly PRINT is supported "+
                                              "but the operation FAILED!" );
                          System.err.println( "Tried but failed to open the file" );
                          x.printStackTrace( System.err );
                        }
                      return null;
                    }
                  }.execute();
              }
            } );
          this.popupMenuPwd.addActionListener( new ActionListener()
            {
              public void actionPerformed( final ActionEvent e )
              {
                final Color feedbackFG = feedback.getForeground();
                final Color feedbackBG = feedback.getBackground();
                final boolean isOpaque = feedback.isOpaque();

                feedback.setOpaque( true );
                feedback.setForeground( new Color(125,126,126) );
                feedback.setBackground( new Color(155,155,155) );

                feedback.setText( popupTarget.getPassword() );
                new SwingWorker<Void,Void>()
                  {
                    public Void doInBackground()
                    {
                      try
                        {
                          Thread.sleep( 1500 );
                        }
                      catch( InterruptedException x )
                        {
                        }
                      feedback.setForeground( feedbackFG );
                      feedback.setBackground( feedbackBG );
                      feedback.setOpaque( isOpaque );
                      feedback.setText( "" );
                      return null;
                    }
                  }.execute();
              }
            } );
          this.popupMenuHide.addActionListener( new ActionListener()
            {
              public void actionPerformed( final ActionEvent e )
              {
                final DefaultListModel<Item> model =
                  (DefaultListModel<Item>)matchingFiles.getModel();

                if( allMatchingFiles == null )
                  {
                    allMatchingFiles = new ArrayList<Item>();
                    final int size = model.size();
                    for( int i=0; i<size; i++ )
                      {
                        allMatchingFiles.add( model.getElementAt(i) );
                      }
                  }
                new SwingWorker<Void,Item>()
                  {
                    public void process( final List<Item> items )
                    {
                      for( Item item : items )
                        {
                          model.removeElement( item );
                        }
                    }
                    public Void doInBackground()
                    {
                      publish( popupTarget );
                      return null;
                    }
                  }.execute();
              }
            } );
          this.popupMenuShow.addActionListener( new ActionListener()
            {
              public void actionPerformed( final ActionEvent e )
              {
                if( allMatchingFiles != null )
                  {
                    final DefaultListModel<Item> model =
                      (DefaultListModel<Item>)matchingFiles.getModel();
                    new SwingWorker<Void,Item>()
                      {
                        boolean needsClearing = true;
                        public void process( final List<Item> items )
                        {
                          if( needsClearing )
                            {
                              model.removeAllElements();
                            }
                          for( Item item : items )
                            {
                              model.addElement( item );
                            }
                        }
                        public Void doInBackground()
                        {
                          for( Item item : allMatchingFiles )
                            {
                              publish( item );
                            }
                          allMatchingFiles = null;
                          return null;
                        }
                      }.execute();

                  }
              }
            } );

          this.popupMenu = new JPopupMenu();
          this.popupMenu.add( this.popupMenuOpen );
          this.popupMenu.add( this.popupMenuEdit );
          this.popupMenu.add( this.popupMenuPrint );
          this.popupMenu.addSeparator();
          this.popupMenu.add( this.popupMenuPwd );
          this.popupMenu.addSeparator();
          this.popupMenu.add( this.popupMenuHide );
          this.popupMenu.add( this.popupMenuShow );
        }

      if( popupTarget == null )
        {
          this.popupMenuOpen.setEnabled( false );
          this.popupMenuEdit.setEnabled( false );
          this.popupMenuPrint.setEnabled( false );

          this.popupMenuPwd.setEnabled( false );
          this.popupMenuHide.setEnabled( false );
        }
      else
        {
          // Always attempt to support OPEN
          this.popupMenuOpen.setEnabled( true );

          if( desktop.isSupported(Desktop.Action.EDIT))
            {
              this.popupMenuEdit.setEnabled( false );
              this.popupMenuEdit.setToolTipText( "Unsupported operation" );
            }
          if( desktop.isSupported(Desktop.Action.PRINT))
            {
              this.popupMenuPrint.setEnabled( false );
              this.popupMenuPrint.setToolTipText( "Unsupported operation" );
            }

          if( item.isEncrypted() )
            {
              this.popupMenuPwd.setEnabled( true );
            }
          else
            {
              this.popupMenuPwd.setEnabled( false );
            }
        }

      this.popupMenuShow.setEnabled( allMatchingFiles != null );

      this.popupMenu.show( e.getComponent(),
                           e.getX(),
                           e.getY() );
    }

    void scanning( final Path directory )
    {
      if( first == null )
        {
          first = directory;
        }
      this.feedback.setText( directory.toString() );
    }

    /**
     * The given {@link Path file} matches the search criteria.
     **/
    synchronized void matches( final Path file,
                               final String password )
    {
      new SwingWorker<Void,Item>()
        {
          public void process( final List<Item> items )
          {
            for( Item item : items )
              {
                final DefaultListModel<Item> model =
                  (DefaultListModel<Item>)matchingFiles.getModel();
                model.addElement( item );
                final int count = model.size();
                info.setText( count+" file"+(count==1?"":"s")+" matched:" );
              }
          }
          public Void doInBackground()
          {
            try
              {
                publish( new Item(file,password) );
              }
            catch( Exception x )
              {
                x.printStackTrace();
              }
            return null;
          }
        }.execute();
    }

    synchronized void done()
    {
      if( stopButton.isEnabled() )
        {
          walker.stop();
        }
      stopButton.setEnabled( false );
      final PasswordManager pwMan = PasswordManager.getInstance();
      final int waitingCount = pwMan.getWaitingCount();
      if( waitingCount > 0 )
        {
          if( forcePasswordManager )
            {
              forcePasswordManager = false;
              pwMan.setVisible( true );
            }
          this.feedback.setText( "Waiting on "+waitingCount+" encrypted "+
                                 "(password-protected) document to be "+
                                 "unlocked" );
        }
      else
        {
          if( stopped )
            {
              this.feedback.setText( "Canceled scan of "+first );
            }
          else
            {
              this.feedback.setText( "Finished scanning "+first );
            }
        }
      this.feedback.setFont( originalFeedbackFont );
    }

    synchronized void refreshWaiting()
    {
      if( ! stopButton.isEnabled() )
        {
          done(); // update the message
        }
    }

    private class Item
    {
      Item( final Path file,
            final String password )
      {
        super();
        this.file = file;
        this.password = password;
      }
      Path getFile()
      {
        return file;
      }
      boolean isEncrypted()
      {
        return (password != null);
      }
      String getPassword()
      {
        return password;
      }
      private final Path file;
      private final String password;
    }

    private class ItemRenderer
      extends JLabel
      implements ListCellRenderer<Item>
    {
      ItemRenderer()
        {
          super();
          setOpaque( true );
        }
      @Override
      public Component getListCellRendererComponent( final JList<? extends Item> list,
                                                     final Item item,
                                                     final int rowIndex,
                                                     final boolean isSelected,
                                                     final boolean hasFocus )
      {
        setIcon( item.isEncrypted() ? lockedIcon : plainIcon );
        setText( item.getFile().toString() );
        if( isSelected )
          {
            Color fg;
            Color bg;
            if( hasFocus )
              {
                if( focusFG == null )
                  {
                    fg = list.getSelectionForeground();
                    bg = list.getSelectionBackground();
                    final int avgFG = (fg.getRed() + fg.getGreen() + fg.getBlue()) / 3;
                    final int avgBG = (bg.getRed() + bg.getGreen() + bg.getBlue()) / 3;
                    if( avgFG > avgBG )
                      {
                        // foreground is brighter than background,
                        // comparable to white on back background
                        focusFG = fg.brighter();
                        focusBG = bg.darker();
                      }
                    else
                      {
                        // foreground is darker than background,
                        // comparable to black on white background
                        focusFG = fg.darker();
                        focusBG = bg.brighter();
                      }
                    focusFG = fg.brighter();
                    focusBG = bg.darker();
                  }
                fg = focusFG;
                bg = focusBG;
              }
            else
              {
                fg = list.getSelectionForeground();
                bg = list.getSelectionBackground();
              }
            setForeground( fg );
            setBackground( bg );
          }
        else
          {
            if( rowIndex % 2 == 0 )
              {
                setForeground( Color.black );
                setBackground( oddRowColor );
              }
            else
              {
                setForeground( Color.black );
                setBackground( evenRowColor );
              }
          }
        setBorder( hasFocus ? border1 : border0 );
        return this;
      }
      private Color focusFG, focusBG;
      private final Border border0 = BorderFactory.createEmptyBorder(1,1,1,1);
      private final Border border1 = BorderFactory.createLineBorder(Color.black,1);
      private final Color oddRowColor = new Color(220,230,240);
      private final Color evenRowColor = new Color(255,255,255);
      private final ImageIcon lockedIcon = Icons.get( "locked" );
      private final ImageIcon plainIcon  = Icons.get( "plain" );
    }

    private JPopupMenu popupMenu;
    private JMenuItem popupMenuOpen, popupMenuEdit, popupMenuPrint;
    private JMenuItem popupMenuPwd, popupMenuHide, popupMenuShow;
    private Item popupTarget;
    private List<Item> allMatchingFiles;
    //
    private Path first;
    private boolean stopped;
    private Font originalFeedbackFont;
    private Color plainFeedbackBgColor;
    //
    private final Color hintFeedbackBgColor = new Color(222,168,168);
    private final TreeWalker walker;
    private final JButton stopButton, closeButton;
    private final JLabel info;
    private final JLabel feedback;
    private final JList<Item> matchingFiles;
  }

  private static String entityEscape( final String text )
  {
    final StringBuilder sb = new StringBuilder();
    final int len = text.length();
    for( int i=0; i<len; i++ )
      {
        final char c = text.charAt( i );
        if( c == '<' )
          {
            sb.append( "&lt;" );
          }
        else if( c == '>' )
          {
            sb.append( "&gt;" );
          }
        else if( c == '&' )
          {
            sb.append( "&amp;" );
          }
        else if( c == '"' )
          {
            sb.append( "&quot;" );
          }
        else if( c == '\'' )
          {
            sb.append( "&apos;" );
          }
        else if( c > 0x127 )
          {
            sb.append( "&#x" ).append( Integer.toString((int)c,16) ).append( ";" );
          }
        else
          {
            sb.append( c );
          }
      }
    return sb.toString();
  }

  private boolean forcePasswordManager = true;
  private final GUI gui;
  private final JTabbedPane tabs;
}
