package com.ringlord.odfind;

import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.GridLayout;

import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;

import java.io.File;

import java.nio.file.Path;

import java.util.List;
import java.util.ArrayList;

import javax.swing.JPanel;
import javax.swing.JLabel;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JTextField;
import javax.swing.BorderFactory;
import javax.swing.JFileChooser;
import javax.swing.DefaultComboBoxModel;

import javax.swing.text.JTextComponent;
import javax.swing.text.Document;

import javax.swing.event.DocumentListener;
import javax.swing.event.DocumentEvent;

/**
 * Collects search criteria and launches a search operation.
 *
 * @author K. Udo Schuermann
 **/
public class SearchEditor
  extends JPanel
{
  public SearchEditor( final GUI gui,
                       final ResultsPanel results )
  {
    super( new BorderLayout() );
    setBorder( BorderFactory.createEmptyBorder(8,8,8,8) );

    this.gui = gui;
    this.results = results;

    final JTextField query;
    final JCheckBox cSensitive;
    final JCheckBox recurse;
    final JButton search;
    final JButton exit;

    final Form p = new Form();
    final JLabel searchForLabel;
    p.add( searchForLabel=new JLabel("Search for:",JLabel.TRAILING),
           query = new JTextField() );
    p.add( new JLabel(""),
           cSensitive = new JCheckBox("Case sensitive",
                                      Settings.get("search-case-sensitive",false)) );

    searchForLabel.setDisplayedMnemonic(KeyEvent.VK_F);
    searchForLabel.setLabelFor(query);

    cSensitive.setMnemonic(KeyEvent.VK_C);

    final DefaultComboBoxModel<String> rootPaths = new DefaultComboBoxModel<String>();
    // @@@ load the previously used paths

    root = new ComboText( rootPaths );
    root.setEditable( true );
    final java.awt.Component comboBoxEditor = root.getEditor().getEditorComponent();
    if( comboBoxEditor instanceof JTextComponent )
      {
        ((JTextComponent)comboBoxEditor).setFont( query.getFont() );
      }

    final JPanel rootPane = new JPanel( new BorderLayout() );
    final JButton pickRoot;

    pickRoot = new JButton( Icons.get("documents") );
    final JLabel lookWhereLabel;
    p.add( lookWhereLabel=new JLabel("Look where:",JLabel.TRAILING),
           root,
           pickRoot );
    p.add( new JLabel(""),
           recurse = new JCheckBox("Recursive search",
                                   Settings.get("search-recursive",true)) );

    lookWhereLabel.setDisplayedMnemonic(KeyEvent.VK_K);
    lookWhereLabel.setLabelFor(root);

    recurse.setMnemonic(KeyEvent.VK_R);

    // The Search command passes to a Search Thread the result panel
    // where a tab will be created with a progress bar; the search can
    // be paused or aborted, but must be manually closed/deleted.
    final JPanel b = new JPanel( new GridLayout(1,0,2,0) );
    b.add( search = new JButton("Search",Icons.get("start")) );
    b.add( exit   = new JButton("Exit",Icons.get("exit")) ); // or "Quit"
    final JPanel c = new JPanel( new FlowLayout(FlowLayout.RIGHT,2,2) );
    c.add( b );
    c.setBorder( BorderFactory.createEmptyBorder(0,0,0,4) );

    search.setMnemonic(KeyEvent.VK_S);
    exit.setMnemonic(KeyEvent.VK_X);

    search.setEnabled( false );
    final Document searchDoc = query.getDocument();
    searchDoc.addDocumentListener( new DocumentListener()
      {
        public void changedUpdate( final DocumentEvent e )
        {
        }
        public void insertUpdate( final DocumentEvent e )
        {
          search.setEnabled( searchDoc.getLength() > 0 );
        }
        public void removeUpdate( final DocumentEvent e )
        {
          search.setEnabled( searchDoc.getLength() > 0 );
        }
      } );

    pickRoot.setToolTipText( "<html>"+
                             "Pick a root directory to search<br>"+
                             "using a file chooser." );
    query.setToolTipText( "<html>"+
                          "Enter here a simple substring or a<br>"+
                          "regular expression (if you don't know<br>"+
                          "what a regular expression is, then you<br>"+
                          "are not likely to need it ;-)" );
    cSensitive.setToolTipText( "<html>"+
                               "Check this to perform a<br>"+
                               "case-sensitive search." );
    root.setToolTipText( "<html>"+
                         "Enter here a directory whose contents<br>"+
                         "(files) are to be searched." );
    search.setToolTipText( "<html>"+
                           "Begin a search. You may change your<br>"+
                           "search parameters and begin another<br>"+
                           "one; searches will be run in parallel,<br>"+
                           "but running more than one search will<br>"+
                           "slow down the overall searching.<br>"+
                           "<br>"+
                           "Note that you will be prompted for a<br>"+
                           "password whenever an encrypted file<br>"+
                           "is encountered. The search continues<br>"+
                           "as you enter passwords." );
    recurse.setToolTipText( "<html>"+
                            "Check this to search all files<br>"+
                            "in all subdirectories, too." );

    final ActionListener beginSearch = new ActionListener()
      {
        public void actionPerformed( final ActionEvent e )
        {
          root.finishEditing(); // ensure that edit action is complete
          search( query.getText().trim(),
                  TreeWalker.QueryType.AUTO_DETECT,
                  cSensitive.isSelected(),
                  new File((String)root.getSelectedItem()).toPath(),
                  recurse.isSelected() );
        }
      };

    query.addActionListener( beginSearch );
    search.addActionListener( beginSearch );
    exit.addActionListener( new ActionListener()
      {
        public void actionPerformed( final ActionEvent e )
        {
          gui.shutdown();
        }
      } );
    pickRoot.addActionListener( new ActionListener()
      {
        public void actionPerformed( final ActionEvent e )
        {
          if( d == null )
            {
              d = new JFileChooser();
              d.setApproveButtonText( "Select" );
              d.setApproveButtonMnemonic( 'S' );
              d.setFileSelectionMode( JFileChooser.DIRECTORIES_ONLY );
              d.setMultiSelectionEnabled( false );
              try
                {
                  final String curDir = (String)root.getSelectedItem();
                  d.setSelectedFile( new File(curDir) );
                }
              catch( Exception x )
                {
                  // oh well, no explicit default, that's okay
                }
            }
          final int result = d.showOpenDialog( pickRoot );
          if( result == JFileChooser.APPROVE_OPTION )
            {
              final File file = d.getSelectedFile();
              if( file != null ) // sanity check
                {
                  root.add( file.toString() );
                }
            }
        }
        private JFileChooser d;
      } );

    add( p, BorderLayout.CENTER );
    add( c, BorderLayout.SOUTH );

    init();
  }

  public void search( final String query,
                      final boolean isCaseSensitive,
                      final Path root )
  {
    search( query,
            TreeWalker.QueryType.AUTO_DETECT,
            isCaseSensitive,
            root,
            true );
  }

  public void search( final String query,
                      final TreeWalker.QueryType queryType,
                      final boolean isCaseSensitive,
                      final Path root,
                      final boolean isRecursive )
  {
    final TreeWalker w = new TreeWalker( results );
    // NOTE: We may never really need user-selectable query type
    // override; until then a hard-coded AUTO_DETECT will do.
    w.setQuery( query,
                queryType,
                isCaseSensitive );
    w.setRoot( root,
               isRecursive );
    w.execute();
  }

  void init()
  {
    final String paths = Settings.get( "search-paths", (String)null );
    if( paths != null )
      {
        final int endOfSeparatorIndex = paths.indexOf(":");
        if( endOfSeparatorIndex > 0 ) // must have at least 1-character separator!
          {
            final String sep = paths.substring( 0, endOfSeparatorIndex );
            final int sepLen = sep.length();
            int start = endOfSeparatorIndex + 1; // +1 is length of ":"
            int pos;
            final DefaultComboBoxModel<String> model = (DefaultComboBoxModel<String>)root.getModel();
            while( (pos = paths.indexOf(sep,start)) > -1 )
              {
                final String path = paths.substring( start, pos );
                if( path.length() > 0 )
                  {
                    model.addElement( path );
                  }
                start = (pos + sepLen);
              }
            final String path = paths.substring( start );
            if( path.length() > 0 )
              {
                model.addElement( path );
              }
          }

        final String activeItem = Settings.get( "active-path", (String)null );
        if( activeItem != null )
          {
            root.setSelectedItem( activeItem );
          }
      }
  }
  void shutdown()
  {
    final DefaultComboBoxModel<String> model = (DefaultComboBoxModel<String>)root.getModel();
    final List<String> paths = new ArrayList<String>();
    final int max = Math.min( Settings.get("directory-count-to-remember",10),
                              model.getSize() - 1 ); // -1 to skip first "" default entry
    for( int i=1; i<=max; i++ )
      {
        paths.add( model.getElementAt(i) );
      }
    final String sep = "\n\t\n";
    final StringBuilder sb = new StringBuilder();
    sb.append( sep ).append( ":" );
    boolean isFirst = true;
    for( String path : paths )
      {
        if( isFirst )
          {
            isFirst = false;
          }
        else
          {
            sb.append( sep );
          }
        sb.append( path );
      }
    Settings.put( "search-paths", sb.toString() );

    final String activePath = (String)root.getSelectedItem();
    Settings.put( "active-path", activePath );
  }

  private final ComboText root;
  private GUI gui;
  private ResultsPanel results;
}
