package com.ringlord.odfind;

import java.awt.Rectangle;

import java.util.prefs.Preferences;

/**
 * Provides an easy interface to the persistent settings for ODFind.
 *
 * @author K. Udo Schuermann
 **/
public class Settings
{
  public static void put( final String key,
                          final String value )
  {
    if( value == null )
      {
        prefs().remove( key );
      }
    else
      {
        prefs().put( key, value );
      }
  }
  public static String get( final String key,
                            final String defaultValue )
  {
    return prefs().get( key, defaultValue );
  }
  // ======================================================================
  public static void put( final String key,
                          final Rectangle rect )
  {
    if( rect == null )
      {
        prefs().remove( key );
      }
    else
      {
        prefs().put( key,
                     rect.x+","+rect.y+","+rect.width+","+rect.height );
      }
  }
  public static Rectangle get( final String key,
                               final Rectangle defaultValue )
  {
    final String value = prefs().get( key, null );
    if( value == null )
      {
        return defaultValue;
      }

    try
      {
        int prev, comma;

        comma = value.indexOf(",");
        int x = Integer.parseInt( value.substring(0,comma) );

        prev = comma + 1;
        comma = value.indexOf(",",prev);
        int y = Integer.parseInt( value.substring(prev,comma) );

        prev = comma + 1;
        comma = value.indexOf(",",prev);
        int w = Integer.parseInt( value.substring(prev,comma) );

        prev = comma + 1;
        int h = Integer.parseInt( value.substring(prev) );

        return new Rectangle( x, y, w, h );
      }
    catch( Exception x )
      {
        x.printStackTrace();
        return defaultValue;
      }
  }
  // ======================================================================
  public static void put( final String key,
                          final Boolean value )
  {
    if( value == null )
      {
        prefs().remove( key );
      }
    else
      {
        prefs().putBoolean( key, value );
      }
  }
  public static boolean get( final String key,
                             final boolean defaultValue )
  {
    return prefs().getBoolean( key, defaultValue );
  }
  // ======================================================================
  public static void put( final String key,
                          final Integer value )
  {
    if( value == null )
      {
        prefs().remove( key );
      }
    else
      {
        prefs().putInt( key, value );
      }
  }
  public static int get( final String key,
                         final int defaultValue )
  {
    return prefs().getInt( key, defaultValue );
  }
  // ======================================================================
  private static synchronized Preferences prefs()
  {
    if( prefs == null )
      {
        prefs = Preferences.userNodeForPackage( com.ringlord.odfind.Main.class );
      }
    return prefs;
  }

  // Hide the constructor
  private Settings()
  {
  }

  private static Preferences prefs;
}
