package com.ringlord.odfind;

import java.awt.FlowLayout;
import java.awt.Component;
import java.awt.Insets;

import javax.swing.JPanel;
import javax.swing.JLabel;
import javax.swing.JButton;
import javax.swing.JTabbedPane;
import javax.swing.ImageIcon;
import javax.swing.BorderFactory;

/**
 * <p>A tab title includes an icon, the title text, and a button that
 * closes (remove) the tab when it is clicked.</p>
 *
 * <pre>
 *   // Add a component to the tabbbed pane but create no title for it (yet)
 *   addTab( null,
 *           tabContent );
 *   int index = getTabCount() - 1;
 *
 *   // create a close button; you are invited to make your own
 *   TabTitle.CloseButton close = TabTitle.createCloseButton( tabs,
 *                                                            tabContent );
 *   // create a title component
 *   TabTitle tabTitle = new TabTitle( titleText,
 *                                     imageIcon,
 *                                     close );
 *   // set the title component for the tab we just created
 *   setTabComponentAt( index,
 *                      tabTitle );
 *
 *   // optional: set a tool tip text, too
 *   setToolTipTextAt( index,
 *                     toolTipText );
 * </pre>
 **/
class TabTitle
  extends JPanel
{
  TabTitle( final String title,
            final ImageIcon icon,
            final JButton close )
  {
    super( new FlowLayout(FlowLayout.LEFT,5,0) );
    setOpaque( false );

    if( icon != null )
      {
        final JLabel l = new JLabel( icon );
        l.setOpaque( false );
        add( l );
      }

    if( title != null )
      {
        final JLabel l = new JLabel( title );
        l.setOpaque( false );
        add( l );
      }

    if( close != null )
      {
        add( close );
      }
  }
  public static CloseButton createCloseButton( final JTabbedPane tabs,
                                               final Component component )
  {
    return new CloseButton( tabs, component );
  }

  static class CloseButton
    extends JButton
  {
    CloseButton( final JTabbedPane tabs,
                 final Component component )
    {
      super( Icons.get("close") );

      // Reduce the current insets; the setMargin operation appears to
      // be an addition to existing values
      final Insets n = getInsets();
      final int min = Math.min( Math.min(n.top,n.bottom),
                                Math.min(n.left,n.right) );
      setMargin( new Insets((min-1) - n.top,
                            min - n.left,
                            (min-1) - n.bottom,
                            min - n.right) );
      this.tabs = tabs;
      this.component = component;
    }
    JTabbedPane getTabs()
    {
      return tabs;
    }
    Component getComponent()
    {
      return component;
    }
    private final JTabbedPane tabs;
    private final Component component;
  }
}
